@@ -7,21 +7,20 @@ import time
 from collections import OrderedDict
 from pathlib import Path
 from subprocess import PIPE, run
-from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
+from typing import Callable, Dict, Iterable, List, Optional, Union
 
 from jinja2 import Environment, FileSystemLoader
+from rich.progress import BarColumn, MofNCompleteColumn, Progress, TextColumn, TimeRemainingColumn
 
-from .logger import logger
-
-# TODO: Add a progress bar
+from autosbatch.logger import logger
 
 
 class SlurmPool:
     """A class for submitting jobs to Slurm."""
 
-    # TODO: support context manager
     time_now = datetime.datetime.now().strftime('%m%d%H%M%S')
-    FILE_DIR = f'./.autosbatch/{time_now}'
+    dir_path = '.autosbatch'
+    FILE_DIR = f'{dir_path}/{time_now}'
     LOG_DIR = f'{FILE_DIR}/log'
     SCRIPTS_DIR = f'{FILE_DIR}/scripts'
 
@@ -36,6 +35,7 @@ class SlurmPool:
         partition: Optional[str] = None,
         max_pool_size: int = 1000,
     ):
+        """Initialize a SlurmPool object."""
         self.ncpus_per_job = ncpus_per_job
 
         self.nodes = self.get_nodes()
@@ -53,7 +53,7 @@ class SlurmPool:
         self._set_pool_size(pool_size=pool_size, max_pool_size=max_pool_size)
 
     @classmethod
-    def get_nodes(cls, sortByload=True) -> OrderedDict:
+    def get_nodes(cls, sortByload=True) -> Dict:
         """Get nodes information from sinfo."""
         command = ['sinfo', '-o', '"%n %e %m %a %c %C %O %R %t"']
         result = run(command, stdout=PIPE, stderr=PIPE, universal_newlines=True)
@@ -76,8 +76,8 @@ class SlurmPool:
                 'state': state,
             }
         if sortByload:
-            nodes = OrderedDict(
-                sorted(nodes.items(), key=lambda x: (x[1]['load'], x[1]['used_mem'], x[1]['used_cpus']))
+            nodes = dict(
+                OrderedDict(sorted(nodes.items(), key=lambda x: (x[1]['load'], x[1]['used_mem'], x[1]['used_cpus'])))
             )
         return nodes
 
@@ -95,7 +95,7 @@ class SlurmPool:
         states: List[str] = ['idle', 'mix'],
         node_list: Optional[List[str]] = None,
         partition: Optional[str] = None,
-    ) -> dict:
+    ) -> None:
         """Get available nodes."""
         if node_list:
             self.nodes = {k: self.nodes[k] for k in node_list if k in self.nodes}
@@ -130,8 +130,8 @@ class SlurmPool:
 
     def _set_pool_size(
         self,
+        max_pool_size: int,
         pool_size: Optional[int] = None,
-        max_pool_size: Optional[int] = None,
     ):
         """Set pool_size."""
         max_pool_size = min(sum(self.jobs_on_nodes.values()), max_pool_size)
@@ -149,16 +149,20 @@ class SlurmPool:
         partition: str,
         node: str,
         cpus_per_task: int,
-        cmds: List[str],
-        job_name: str = 'aotusbatch',
+        cmds: Union[str, List[str]],
+        job_name: str = 'job',
         logging_level: int = logging.WARNING,
     ):
+        """Submit a single job."""
         logger.setLevel(logging_level)
         Path(cls.SCRIPTS_DIR).mkdir(parents=True, exist_ok=True)
         Path(cls.LOG_DIR).mkdir(parents=True, exist_ok=True)
         templateLoader = FileSystemLoader(searchpath=f"{os.path.dirname(os.path.realpath(__file__))}/template")
         env = Environment(loader=templateLoader)
         template = env.get_template(cls.CPU_OpenMP_TEMPLATE)
+
+        if isinstance(cmds, str):
+            cmds = [cmds]
         output_from_parsed_template = template.render(
             job_name=job_name,
             partition=partition,
@@ -176,6 +180,7 @@ class SlurmPool:
         result = run(command, stdout=PIPE, stderr=PIPE, universal_newlines=True)
         slurm_id = result.stdout.strip().split()[-1]
         logger.info(f'Sumbitted Task: {job_name} to {node}, containing {len(cmds)} jobs. Slurm ID: {slurm_id}')
+        logger.debug(f'Commands: {cmds}')
 
     def multi_submit(
         self,
@@ -185,6 +190,7 @@ class SlurmPool:
         shuffle: bool = False,
         sleep_time: float = 0.5,
     ):
+        """Submit jobs to multiple nodes."""
         if shuffle:
             import random
 
@@ -208,58 +214,67 @@ class SlurmPool:
                 used_nodes[k] -= registed_jobs - self.pool_size
                 break
         logger.info(f'Used {len(used_nodes)} nodes.')
-        logger.info(f'Each node will excute {self.max_jobs_per_node} tasks in parallel.')
+        logger.info(f'Each node will excute {max(used_nodes.values())} tasks in parallel.')
         logger.info(f'{used_nodes}')
         k, m = divmod(len(cmds), self.pool_size)
         ith = 0
         task_log = {}
-        for node, n_jobs in used_nodes.items():
-            logger.info(f'{node}: {n_jobs} tasks')
-            for _ in range(n_jobs):
-                time.sleep(sleep_time)
-                start, end = ith * k + min(ith, m), (ith + 1) * k + min(ith + 1, m)
-                logger.info(f'Task {ith}: containing job {start}-{end-1}')
-                task_name = f'{job_name}_{ith:>03}'
-                self.single_submit(
-                    self.nodes[node]['partition'],
-                    node,
-                    self.ncpus_per_job,
-                    cmds[start:end],
-                    task_name,
-                    logging_level,
-                )
-                ith += 1
-                task_log[task_name] = {
-                    'node': node,
-                    'script': f'{task_name}.sh',
-                    'stdout': f'{task_name}.out.log',
-                    'stderr': f'{task_name}.err.log',
-                    'cmd': cmds[start:end],
-                }
+        with Progress(
+            TextColumn("{task.description}"), BarColumn(), MofNCompleteColumn(), TimeRemainingColumn()
+        ) as progress:
+            for node, n_jobs in used_nodes.items():
+                logger.info(f'{node}: {n_jobs} tasks')
+                task = progress.add_task(f"Submitting to {node}...", total=n_jobs)
+                for _ in range(n_jobs):
+                    time.sleep(sleep_time)
+                    start, end = ith * k + min(ith, m), (ith + 1) * k + min(ith + 1, m)
+                    logger.info(f'Task {ith}: containing job {start}-{end-1}')
+                    task_name = f'{job_name}_{ith:>03}'
+                    self.single_submit(
+                        self.nodes[node]['partition'],
+                        node,
+                        self.ncpus_per_job,
+                        cmds[start:end],
+                        task_name,
+                        logging_level,
+                    )
+                    progress.update(task, advance=1)
+                    ith += 1
+                    task_log[task_name] = {
+                        'node': node,
+                        'script': f'{task_name}.sh',
+                        'stdout': f'{task_name}.out.log',
+                        'stderr': f'{task_name}.err.log',
+                        'cmd': cmds[start:end],
+                    }
         with open(f'{self.FILE_DIR}/{self.time_now}.log', 'w') as f:
             logger.info(f'Writing task log to {self.FILE_DIR}/{self.time_now}.log')
             json.dump(task_log, f, indent=4)
 
     def starmap(self, func: Callable, params: Iterable[Iterable]):
-        """
-        func: function
-        params: list of tuple
-        return: None
-        """
+        """Submit a list of commands to the cluster."""
         cmds = [func(*i) for i in params]
         self.multi_submit(cmds, func.__name__)
 
     def map(self, func: Callable, params: Iterable):
-        """
-        func: function
-        params: list
-        return: None
-        """
+        """Submit a list of commands to the cluster."""
         cmds = [func(i) for i in params]
         self.multi_submit(cmds, func.__name__)
 
     @classmethod
     def clean(cls):
-        """Clean up the scripts and log files"""
-        command = ['rm', '-rf', cls.FILE_DIR]
+        """Clean up the scripts and log files."""
+        command = ['rm', '-rf', cls.dir_path]
         _ = run(command, stdout=PIPE, stderr=PIPE, universal_newlines=True)
+
+    def close(self):
+        """Clean up the scripts and log files."""
+        pass
+
+    def __enter__(self):
+        """Clean up the scripts and log files."""
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        """Clean up the scripts and log files."""
+        self.close()
